import 'dart:io';

import 'package:flutter/foundation.dart';
import 'package:flutter_base_loader/model/deviceInfoJsonable.dart';
import 'package:flutter_base_loader/model/packageInfoJsonable.dart';
import 'package:flutter_base_loader/model/platformInfoJsonable.dart';
import 'package:package_info_plus/package_info_plus.dart';
import 'package:device_info_plus/device_info_plus.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'package:uuid/uuid.dart';

class baseInfo {
  static late packageInfoJsonable package;
  static late platformInfoJsonable platform;
  static late deviceInfoJsonable device;

  static Future<void> init() async {
    await _initPackageInfo();
    await _initPlatformInfo();
    await _initDeviceInfo();
    return Future.value();
  }

  static Map<String, dynamic> toJson() {
    final Map<String, dynamic> data = new Map<String, dynamic>();
    data['package'] = package.toJson();
    data['platform'] = platform.toJson();
    data['device'] = device.toJson();
    return data;
  }

  //_initPackageInfo internal method
  static Future<packageInfoJsonable> _initPackageInfo() async {
    var _packageInfo = await PackageInfo.fromPlatform();
    package = packageInfoJsonable(
      appName: _packageInfo.appName,
      packageName: _packageInfo.packageName,
      version: _packageInfo.version,
      buildNumber: _packageInfo.buildNumber,
      buildSignature: _packageInfo.buildSignature,
    );
    return Future.value(package);
  }

  static Future<platformInfoJsonable> _initPlatformInfo() async {
    platform = platformInfoJsonable(
      operatingSystem: kIsWeb ? "web" : Platform.operatingSystem,
      operatingSystemVersion: kIsWeb ? "" : Platform.operatingSystemVersion,
      dartVersion: kIsWeb ? "" : Platform.version,
      numberOfProcessors: kIsWeb ? 0 : Platform.numberOfProcessors,
      localHostname: kIsWeb ? "" : Platform.localHostname,
    );
    return Future.value(platform);
  }

  static Future<deviceInfoJsonable> _initDeviceInfo() async {
    final key = "system_guid";

    final prefs = await SharedPreferences.getInstance();
    final _info = DeviceInfoPlugin();

    var guid = prefs.getString(key) ?? "";
    var computerName = "";
    var model = "";
    if (guid.isEmpty) {
      if (kIsWeb) {
        final _data = await _info.webBrowserInfo;
        // guid = _data ?? "";
        computerName = _data.browserName.toString();
        model = _data.browserName.toString();
      } else if (Platform.isAndroid) {
        final _data = await _info.androidInfo;
        guid = _data.androidId ?? "";
        computerName = _data.host ?? "";
        model = _data.model ?? "";
        // _data.device
      } else if (Platform.isIOS) {
        final _data = await _info.iosInfo;
        guid = _data.identifierForVendor ?? "";
        computerName = _data.name ?? "";
        model = _data.model ?? "";
      } else if (Platform.isLinux) {
        final _data = await _info.linuxInfo;
        guid = _data.machineId ?? "";
        model = _data.prettyName;
        // hostname=_data.
      } else if (Platform.isMacOS) {
        final _data = await _info.macOsInfo;
        guid = _data.systemGUID ?? "";
        computerName = _data.computerName;
        model = _data.model;
      } else if (Platform.isWindows) {
        final _data = await _info.windowsInfo;
        computerName = _data.computerName;
        model = _data.computerName;
      }
    }
    //set guid as random
    if (guid.isEmpty) {
      guid = Uuid().v4();
      await prefs.setString(key, guid);
    }
    device = deviceInfoJsonable(
      guid: guid,
      computerName: computerName,
      model: model,
    );
    return Future.value(device);
  }
}
